/* mlm_airtools.c
 *
 * CallCommander Integration for an AirTools 6100 Broadcast Audio Delay
 *
 *   (C) Copyright 2002-2007 Fred Gleason <fredg@paravelsystems.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This module displays call screening data on the LCD displays of
 * a Logitek Remora control surface.
 *
 * To compile this module, just do:
 * 
 *   gcc -shared -o mlm_airtools.mlm mlm_airtools.c
 */

/*
 * Site Definitions -- Customize these values for your site!
 */
#define DELAY_TTY_DEVICE "/dev/ttyS5"
#define DELAY_TTY_SPEED 38400
#define DELAY_UNIT_ADDRESS 1

/*
 * **************************************************************************
 *  You should not need to change anything below here!
 * **************************************************************************
 */

#define FLASH_TIME_ON 600
#define FLASH_TIME_OFF 400

#include <stdio.h>
#include <string.h>
#include <mlm/mlm.h>

/*
 * Globals
 */
int delay_offset;
int delay_handle;
int delay_msg_istate;

void MLMStartup()
{
  int i;
  int j;

  /*
   * Initilaize the Audio Delay Connection
   */
  if(strlen(DELAY_TTY_DEVICE)!=0) {
    delay_offset=0;
    delay_msg_istate=0;
    delay_handle=MLMOpenSerial(DELAY_TTY_DEVICE,DELAY_TTY_SPEED,0,8);
    MLMStartTimer(0,1000,MLM_TIMER_REPEATING);
  }
}


void MLMDumpStateChanged(int state)
{
  char buffer[10];

  if(state) {
//    sprintf(buffer,"\xFB%c\x00\x03\x90\x08\x65",0xFF&DELAY_UNIT_ADDRESS);
    strcpy(buffer,"\xFB\x01\x00\x03\x90\x08\x65");
    MLMSendSerial(delay_handle,"\xFB\x01\x00\x03\x90\x08\x65",7);
  }
  else {
    MLMSendSerial(delay_handle,"\xFB\x01\x00\x03\x90\x00\x6D",7);
  }
}


void MLMMetadataUpdateRequested()
{
  MLMSendDelayOffset(100*delay_offset);
}


void MLMTimerExpired(int timernum)
{
  /*
   * Query Delay Length
   */
  MLMSendSerial(delay_handle,"\xFB\x01\x00\x02\x11\xEB",6);
}


void MLMSerialReceived(int handle,const char *data,int len)
{
  int i;

  for(i=0;i<len;i++) {
    switch(delay_msg_istate) {
	case 0:  /* Unit Address */
	  if(data[i]==DELAY_UNIT_ADDRESS) {
	    delay_msg_istate=1;
	  }
	  break;

	case 1:  /* Device Type */
	  delay_msg_istate=2;
	  break;

	case 2:  /* Mfgr's Code */
	  delay_msg_istate=3;
	  break;

	case 3:  /* Message Length MSB */
	  if(data[i]==0x00) {
	    delay_msg_istate=4;
	  }
	  else {
	    delay_msg_istate=0;
	  }
	  break;

	case 4:  /* Message Length LSB */
	  if(data[i]==0x0C) {
	    delay_msg_istate=5;
	  }
	  else {
	    delay_msg_istate=0;
	  }
	  break;

	case 5:  /* Make up percentage (0.5% steps, 0 - 200) */
	  delay_msg_istate=6;
	  break;

	case 6:  /* Make up time (0.1 sec steps, 0 - 200) */
	  if((0xFF&data[i])!=delay_offset) {
	    delay_offset=0xFF&data[i];
	    MLMSendDelayOffset(100*delay_offset);
	  }
	  delay_msg_istate=0;
	  break;
    }
  }
}
